import plotly.graph_objs as go
import pandas as pd


class Graphics:

    def __init__(self, data_group, status, width=None, height=None):
        self.data = data_group.sort_index()
        self.status = status
        self.width = width
        self.height = height

    def plot(self, metric, line=None, color=None):
        bandxaxis = go.layout.XAxis(title="Ano")
        bandyaxis = go.layout.YAxis(title="")

        layout = dict(title=metric.title(),
                      width=self.width, height=self.height,
                      xaxis=bandxaxis, yaxis=bandyaxis,
                      font=dict(family='Time New Roman', color='rgb(0,0,0)')
                      )

        data = list()
        data.append(self._plot_iha(metric, color))
        try:
            for i in line:
                if i != 'median_line':
                    data.append(self._plot_rva(line=line[i], metric=metric))
        except:
            pass

        fig = dict(data=data, layout=layout)
        return fig, data

    def _plot_rva(self, metric, line):
        line_graph = go.Scatter(x=self.data.index,
                                y=[line[metric]] * len(self.data.index),
                                name='RVA_'+line.name.split('_')[0],
                                mode='lines',
                                line=dict(width=1, color='rgb(0, 0, 0)'),
                                opacity=1, connectgaps=False)
        return line_graph

    def _plot_iha(self, metric, color):
        point = go.Scatter(x=self.data.index,
                           y=self.data[metric].loc[self.data.index].values,
                           name=self.status.title()+'-impacto',
                           mode='markers+lines',
                           marker=dict(size=8, color=color, line=dict(width=1, color=color)))

        return point



import numpy as np

from hydrocomp import statistic as e
from hydrocomp.statistic.stats_build import StatsBuild
from scipy.stats import pearson3
from lmoments3.distr import pe3


class Pearson3(StatsBuild):

    name = 'Pearson3'
    estimator = None
    parameter = {'loc': None, 'scale': None}

    def __init__(self, data=None,  loc=None, scale=None):
        self.loc = loc
        self.scale = scale
        self.parameter['loc'] = self.loc
        self.parameter['scale'] = self.scale
        super().__init__(data, loc, scale)
        self.dist = pearson3(loc=self.loc, scale=self.scale, skew=0.1)

    def mml(self):
        if self.data is None:
            raise e.DataNotExist("Data not's None", 25)
        mml = pe3.lmom_fit(self.data)
        self.estimador = 'MML'
        self.loc = mml['loc']
        self.scale = mml['scale']
        self.dist = pearson3(loc=self.loc, scale=self.scale, skew=0.1)
        return self.loc, self.scale

    def mvs(self):
        if self.data is None:
            raise e.DataNotExist("Data not's None", 35)
        mvs = pearson3.fit(data=self.data)
        self.estimador = 'MVS'
        self.loc = mvs[0]
        self.scale = mvs[1]
        self.dist = pearson3(loc=self.loc, scale=self.scale, skew=0.1)

        return self.loc, self.scale



from unittest import TestCase
from hydrocomp.statistic.genextre import Gev
from hydrocomp.statistic.genpareto import Gpa


class TestGev(TestCase):

    data = [1347,  857, 1626,  977, 1065,  997,  502, 1663,  992, 
            1487, 1041, 2251, 1110, 1553, 1090, 1268, 1113, 1358,  402]
    dist = Gev(data=data)
    
    def test_dist(self):
        name = 'GEV'
        self.assertEquals(self.dist.name, name, 'Name: GEV')

    def test_mml(self):
        mml = (0.14684253029124203, 1023.9891165624797, 380.3053838205217)
        self.assertEquals([self.dist.mml(), self.dist.estimador], [mml, 'MML'], 'Fit_MML: %s, %s, %s' % mml)

    def test_mvs(self):
        mvs = (-5.83785197466355, 403.3270953313672, 7.747500635081945)
        self.assertEquals([self.dist.mvs(), self.dist.estimador], [mvs, 'MVS'], 'Fit_MVS: %s, %s, %s' % mvs)

    def test_prob(self):
        prob_mml = 0.7781690064347855
        prob_mvs = 0.7287813740394129
        self.dist.mml()
        self.assertEquals(self.dist.probs(1500), prob_mml, 'Prob: %s' % prob_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.probs(1500), prob_mvs, 'Prob: %s' % prob_mvs)

    def test_value(self):
        value_mml = 1456.9948303470273
        value_mvs = 2314.9143444142505
        self.dist.mml()
        self.assertEquals(self.dist.values(0.75), value_mml, 'Value: %s' % value_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.values(0.75), value_mvs, 'Value: %s' % value_mvs)
    
    def test_values(self):
        value_mvs = [2314.9143444142505, 413.27574336098405]
        value_mml = [1456.9948303470273, 1159.6914703217076]
        self.dist.mml()
        self.assertEquals(self.dist.values([0.75, 0.5]), value_mml, 'Value: %s' % value_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.values([0.75, 0.5]), value_mvs, 'Value: %s' % value_mvs)

    def test_probs(self):
        prob_mml = [0.7781690064347855, 0.34479635611222237]
        prob_mvs = [0.7287813740394129, 0.7039216570017871]
        self.dist.mml()
        self.assertEquals(self.dist.probs([1500, 1000]), prob_mml, 'Prob: %s' % prob_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.probs([1500, 1000]), prob_mvs, 'Prob: %s' % prob_mvs)

    def test_interval(self):
        ic_mvs = (402.00217396627875, 45018159.2649536)
        ic_mml = (571.2282612439494, 1939.4616813678326)
        self.dist.mml()
        self.assertEquals(self.dist.interval(0.9), ic_mml, 'Value: (%s, %s)' % ic_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.interval(0.9), ic_mvs, 'Value: (%s, %s)' % ic_mvs)


class TestGpa(TestCase):

    data = [1347,  857, 1626,  977, 1065,  997,  502, 1663,  992, 
            1487, 1041, 2251, 1110, 1553, 1090, 1268, 1113, 1358,  402]
    dist = Gpa(data=data)
    
    def test_dist(self):
        name = 'GPA'
        self.assertEquals(self.dist.name, name, 'Name: GPA')

    def test_mml(self):
        mml = (-0.7072859839251329, 560.8626486522879, 1082.1146688970641)
        self.assertEquals([self.dist.mml(), self.dist.estimador], [mml, 'MML'], 'Fit_MML: %s, %s, %s' % mml)

    def test_mvs(self):
        mvs = (-1.1982244351093645, -6.282925274294001, 2704.731558018805)
        self.assertEquals([self.dist.mvs(), self.dist.estimador], [mvs, 'MVS'], 'Fit_MVS: %s, %s, %s' % mvs)

    def test_prob(self):
        prob_mml = 0.7395295673854643
        prob_mvs = 0.6008635213747953
        self.dist.mml()
        self.assertEquals(self.dist.probs(1500), prob_mml, 'Prob: %s' % prob_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.probs(1500), prob_mvs, 'Prob: %s' % prob_mvs)

    def test_value(self):
        value_mml = 1516.8984252482405
        value_mvs = 1822.2708593345496
        self.dist.mml()
        self.assertEquals(self.dist.values(0.75), value_mml, 'Value: %s' % value_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.values(0.75), value_mvs, 'Value: %s' % value_mvs)
    
    def test_values(self):
        value_mvs = [1822.2708593345496, 1267.2505558875055]
        value_mml = [1516.8984252482405, 1153.7636259098695]
        self.dist.mml()
        self.assertEquals(self.dist.values([0.75, 0.5]), value_mml, 'Value: %s' % value_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.values([0.75, 0.5]), value_mvs, 'Value: %s' % value_mvs)

    def test_probs(self):
        prob_mml = [0.7395295673854643, 0.3801780483231015]
        prob_mvs = [0.6008635213747953, 0.3889507274477421]
        self.dist.mml()
        self.assertEquals(self.dist.probs([1500, 1000]), prob_mml, 'Prob: %s' % prob_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.probs([1500, 1000]), prob_mvs, 'Prob: %s' % prob_mvs)

    def test_interval(self):
        ic_mvs = (128.27430967073587, 2188.675318921949)
        ic_mml = (615.3731031559984, 1906.9600907224512)
        self.dist.mml()
        self.assertEquals(self.dist.interval(0.9), ic_mml, 'Value: (%s, %s)' % ic_mml)
        self.dist.mvs()
        self.assertEquals(self.dist.interval(0.9), ic_mvs, 'Value: (%s, %s)' % ic_mvs)


"""
Created on 21 de mar de 2018

@author: clebson
"""

from hydrocomp.files.fileRead import FileRead


class Chesf(FileRead):

    extension = "xls"
    source = "CHESF"

    def __init__(self):
        pass



class StationError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "StationError: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")



import pandas as pd
import plotly as py
import scipy.stats as stat
from hydrocomp.statistic.genextre import Gev

from hydrocomp.graphics.genextreme import GenExtreme
from hydrocomp.graphics.hydrogram_annual import HydrogramAnnual
from hydrocomp.graphics.polar import Polar


class Maximum(object):
    distribution = 'GEV'

    def __init__(self, obj, station):
        self.obj = obj
        self.station = station
        self.peaks = self.__annual()
        self.dist_gev = Gev(self.peaks['peaks'].values)

    def __annual(self):
        self.obj.month_start_year_hydrologic()
        data_by_year_hydrologic = self.obj.data.groupby(pd.Grouper(freq=self.obj.month_abr_flood))
        max = data_by_year_hydrologic[self.station].max()
        idx = data_by_year_hydrologic[self.station].idxmax()
        max_vazao = max.values
        idx_vazao = idx.values
        self.peaks = pd.DataFrame(max_vazao, index=idx_vazao, columns=['peaks'])
        return self.peaks

    def period_return(self, magnitude, estimador):
        if estimador == 'MML':
            self.dist_gev.mml()
        elif estimador == 'MVS':
            self.dist_gev.mvs()

        p = self.dist_gev.probs(magnitude)
        return 1/(1-p)

    def magnitude(self, period_return, estimador):
        if estimador == 'MML':
            self.dist_gev.mml()
        elif estimador == 'MVS':
            self.dist_gev.mvs()

        p = 1-(1 / period_return)
        return self.dist_gev.values(p)

    def plot_distribution(self, title, estimador, type_function, save=False):
        if estimador == 'MVS':
            self.dist_gev.mvs()
        elif estimador == 'MML':
            self.dist_gev.mml()
        else:
            raise ValueError("Estimador: [mvs or mml]")
        genextreme = GenExtreme(title, self.dist_gev.shape, self.dist_gev.loc, self.dist_gev.scale)
        data, fig = genextreme.plot(type_function)
        if save:
            py.image.save_as(fig, filename='gráficos/GEV_%s_%s.png' % (type_function,
                                                                       estimador))
        return fig, data

    def hydrogram(self, save=False, width=None, height=None, size_text=16, title=None):
        _hydrogram = HydrogramAnnual(data=self.obj.data[self.station], peaks=self.peaks, width=height, height=width,
                                     size_text=size_text, title=title, station=self.station)
        fig, data = _hydrogram.plot()
        if save:
            py.image.save_as(fig, filename='gráficos/hidrogama_maximas_anuais.png')

        return fig, data

    def polar(self, save=False, width=None, height=None, size_text=14, title="Máximas Anuais"):
        _polar = Polar(df_events=self.peaks)
        fig, data = _polar.plot(width=width, height=height, size_text=size_text, title=title)
        if save:
            py.image.save_as(fig, filename='graficos/polar_maximas_anuais.png')

        return fig, data



"""
Created on 21 de mar de 2018

@author: clebson
"""
import os
import multiprocessing as mp
import pandas as pd
from hydrocomp.files.files import Files
from abc import abstractmethod, ABCMeta


class FileRead(Files, metaclass=ABCMeta):

    def __init__(self, path_file=None, station=None, *args, **kwargs):
        try:
            if os.path.isfile(path_file):
                self.path = os.path.dirname(path_file)
                self.name, ext = os.path.splitext(os.path.basename(path_file))
                self.api = False
                if ext != ".{}".format(self.extension):
                    raise Exception('Formato invalido')
            elif os.path.isdir(path_file):
                self.path = path_file
                self.name = None
                self.api = False
            else:
                self.path = station
                self.name = None
                self.api = True
        except TypeError:
            self.path = station
            self.name = None
            self.api = True
        super().__init__(path_file=self.path)

    @abstractmethod
    def list_files(self):
        return super().list_files()

    @abstractmethod
    def read(self):
        if type(self.name) == list and len(self.name) > 1:
            p = mp.Pool(mp.cpu_count())
            listaDfs = p.map(self.read, self.name)
            p.close()
            if self.source == 'ANA':
                dataFlow = pd.DataFrame()
                for df in listaDfs:
                    if len(df) > 0:
                        dataFlow = dataFlow.combine_first(df)
                    else:
                        dataFlow = dataFlow
                return dataFlow.sort_index()
            else:
                dataFlow = pd.DataFrame(listaDfs)
                return dataFlow.sort_index()
        else:
            return self.read(self.name)



"""
Created on 21 de mar de 2018

@author: clebson
"""
from hydrocomp.files.fileRead import FileRead


class Nasa(FileRead):
    """
    class files read: National Aeronautics and
                    Space Administration - NASA
    """
    source = "NASA"
    extension = "hdf5"
    
    def __init__(self, params):
        pass



from scipy.stats import genpareto
import pandas as pd

from hydrocomp.comparasion.bootstrap_build import BootstrapBuild


class BootsGenPareto(BootstrapBuild):

    def __init__(self, shape, location, scale, size):
        super().__init__(shape, location, scale, size)

    def fit_resample(self):
        resample = genpareto.rvs(self.shape, self.location, self.scale,
                                 self.size)
        return genpareto.fit(resample)

    def fits_resamples(self, quantity):
        list_fits = list()
        for i in range(quantity):
            list_fits.append(self.fit_resample())

        return list_fits

    def magnitudes_resamples(self, quantity):
        dic_magns = {0.001: list(), 0.01: list(), 0.1: list(),
                     0.5: list(), 0.9: list(), 0.99: list(), 0.999: list()
                     }
        for i in range(quantity):
            fit = self.fit_resample()
            for j in dic_magns:
                mag = genpareto.ppf(j, fit[0], fit[1], fit[2])
                dic_magns[j].append(mag)

        return pd.DataFrame(dic_magns)



import plotly.graph_objs as go


class ComparationDistribution(object):

    def __init__(self, datas, name, type_function=None):
        self.datas = datas
        self.type_function = type_function
        self.name = name

    def plot(self):
        bandxaxis = go.layout.XAxis(title="Vazão(m³/s)")
        bandyaxis = go.layout.YAxis(title="")

        layout = dict(title="Probabilidade Acumulada",
                      showlegend=True,
                      width=1890, height=827,
                      xaxis=bandxaxis,
                      yaxis=bandyaxis,
                      font=dict(family='Time New Roman', size=28, color='rgb(0,0,0)'))

        fig = dict(data=self.datas, layout=layout)
        return fig, self.datas



import pandas as pd
import numpy as np
import calendar
import datetime

import plotly as py
import plotly.graph_objs as go


class Gantt(object):

    def __init__(self, data):
        self.data = data

    def get_gantt(self, df, less, index):

        color = 0
        n = 1
        for j in less.index:
            df.at[index, 'Task'] = self.data.name
            df.at[index, 'Description'] = self.data.name + ' - %s' % j
            df.at[index, 'IndexCol'] = color
            df.at[index, 'Start'] = less['Start'].loc[j]
            df.at[index, 'Finish'] = less['Finish'].loc[j]

            color += (100 * n)
            n *= -1
            index += 1
        return df, index

    @staticmethod
    def get_spells(data_peaks, month_water):
        df_spells = pd.DataFrame(columns=['Task', 'Start', 'Finish', 'Complete', 'Name'])
        index = 0
        dates = pd.date_range(start=pd.to_datetime('1/%s/1998' % month_water[0], dayfirst=True), periods=365, freq='D')
        inter = data_peaks['peaks'].max()-data_peaks['peaks'].min()
        if len(data_peaks) > 0:

            for groups in data_peaks.groupby(pd.Grouper(freq=month_water[1])):
                for i in groups[1].index:

                    #df_spells.at[index, 'Complete'] = 100 - (
                    #            (data_peaks['peaks'].max() - data_peaks['peaks'].loc[i]) / data_peaks['peaks'].max()) * 100
                    df_spells.at[index, 'Complete'] = 100-(100*(data_peaks['peaks'].max() - data_peaks['peaks'].loc[i])/inter)
                    #df_spells.at[index, 'Complete'] = data_peaks['peaks'].loc[i]/1000
                    start = data_peaks['Start'].loc[i]
                    end = data_peaks['End'].loc[i]
                    df_spells.at[index, 'Name'] = data_peaks['peaks'].loc[i]
                    df_spells.at[index, 'Task'] = int(groups[0].year)
                    len_days = len(pd.date_range(start, end))
                    for date in dates:

                        if date.month == start.month and date.day == start.day:
                            inter_date = pd.date_range(start=date, periods=len_days)
                            if inter_date[-1] > dates[-1]:
                                date_start = pd.to_datetime(
                                    '%s/%s/%s' % (inter_date[0].day, inter_date[0].month, inter_date[0].year - 1),
                                    dayfirst=True)
                                date_end = pd.to_datetime(
                                    '%s/%s/%s' % (inter_date[-1].day, inter_date[-1].month, inter_date[-1].year - 1),
                                    dayfirst=True)

                                df_spells.at[index, 'Start'] = date_start
                                df_spells.at[index, 'Finish'] = date_end
                            else:

                                df_spells.at[index, 'Start'] = inter_date[0]
                                df_spells.at[index, 'Finish'] = inter_date[-1]
                    index += 1
                else:
                    pass
            #df_spells.to_csv("df_spells.csv")
        return df_spells, index, dates[0], dates[-1]



import scipy.stats as stat
import pandas as pd

import plotly.graph_objs as go

from hydrocomp.graphics.distribution_biuld import DistributionBiuld


class GenPareto(DistributionBiuld):

    def __init__(self, title, shape, location, scale):
        super().__init__(title, shape, location, scale)

    def cumulative(self):
        datas = self._data('cumulative')
        data = [go.Scatter(x=datas['peaks'], y=datas['Cumulative'],
                           name=self.title, line=dict(color='rgb(128, 128, 128)',
                                                      width=2))]

        bandxaxis = go.layout.XAxis(title="Vazão(m³/s)")
        bandyaxis = go.layout.YAxis(title="Probabilidade")

        layout = dict(title="GP - Acumulada: %s" % self.title,
                      showlegend=True,
                      width=945, height=827,
                      xaxis=bandxaxis,
                      yaxis=bandyaxis,
                      font=dict(family='Time New Roman', size=28, color='rgb(0,0,0)')
                      )

        fig = dict(data=data, layout=layout)

        return fig, data

    def density(self):
        datas = self._data('density')
        data = [go.Scatter(x=datas['peaks'], y=datas['Density'],
                           name=self.title, line=dict(color='rgb(128, 128, 128)',
                                                      width=2))]

        bandxaxis = go.layout.XAxis(title="Vazão(m³/s)")
        bandyaxis = go.layout.YAxis(title="")

        layout = dict(title="GP - Densidade: %s" % self.title,
                      showlegend=True,
                      width=945, height=827,
                      xaxis=bandxaxis,
                      yaxis=bandyaxis,
                      font=dict(family='Time New Roman', size=28, color='rgb(0,0,0)')
                      )

        fig = dict(data=data, layout=layout)
        return fig, data

    def _data_density(self):

        cumulative = self._data_cumulative()
        density = stat.genpareto.pdf(cumulative['peaks'].values, self.shape,
                                     loc=self.location, scale=self.scale)

        dic = {'peaks': cumulative['peaks'].values, 'Density': density}

        return pd.DataFrame(dic)

    def _data_cumulative(self):
        probability = list()
        for i in range(1, 1000):
            probability.append(i/1000)

        quantiles = stat.genpareto.ppf(probability, self.shape,
                                       loc=self.location,
                                       scale=self.scale)

        dic = {'peaks': quantiles, 'Cumulative': probability}

        return pd.DataFrame(dic)



"""
Created on 21 de mar de 2018

@author: clebson
"""
import os
import pandas as pd

from hydrocomp.files.fileRead import FileRead


class Cemaden(FileRead):
    """
    class files read: Centro Nacional de Monitoramento e
                    Alertas de Desastres Naturais - CEMADEN
    """
    source = 'Cemaden'
    extension = 'sam'

    def __init__(self, path_file=os.getcwd()):
        super().__init__(path_file)

    def list_files(self):
        pass

    def read(self, name=None):
        if name is None:
            return super().read()
        else:
            self.name = name
            data = self.__read_sam()
            return data

    def __lines_sam(self):
        list_lines = []
        with open(os.path.join(self.path, self.name+'.'+Cemaden.extension), 'r') as \
                arq:
            for line in arq.readlines():
                list_lines.append(line.split())
        return list_lines

    def __read_sam(self):
        list_lines = self.__lines_sam()
        dado = []
        index = []
        data = self.name.split('_')[1].replace('-', '')
        date_hour = pd.to_datetime(data)
        cont = 0
        for line in list_lines:
            if cont > 0:
                dado.append(float(line[3]))
                index.append((line[1], line[2]))
            cont += 1
        se = pd.Series(dado, index=index, name=date_hour)
        return se



import plotly.graph_objs as go
import pandas as pd

from hydrocomp.graphics.hydrogram_build import HydrogramBuild


class HydrogramAnnual(HydrogramBuild):

    def __init__(self, data, peaks, width=None, height=None, size_text=None, title=None, station=None):
        self.data = pd.DataFrame(data)
        self.peaks = peaks
        self.station = station
        super().__init__(width=width, height=height, size_text=size_text, title=title)

    def plot(self):
        bandxaxis = go.layout.XAxis(title="Data")
        bandyaxis = go.layout.YAxis(title="Vazão(m³/s)")

        layout = dict(
            title=dict(text=self.title, x=0.5, xanchor='center', y=0.95, yanchor='top',
                       font=dict(family='Courier New, monospace', size=self.size_text + 10)),
            xaxis=bandxaxis,
            yaxis=bandyaxis,
            width=self.width, height=self.height,
            font=dict(family='Courier New, monospace', size=self.size_text, color='#7f7f7f'),
            showlegend=True, plot_bgcolor='rgba(0,0,0,0)', paper_bgcolor='rgba(0,0,0,0)')

        data = list()
        data.append(self._plot_one(data=self.data, station=self.station, color='rgb(0,0,0)'))
        data.append(self._plot_event_peaks())

        fig = dict(data=data, layout=layout)
        return fig, data

    def _plot_event_peaks(self):
        point_peak = go.Scatter(x=self.peaks['peaks'].index,
                                y=self.peaks['peaks'].values,
                                name="Pico",
                                mode='markers',
                                marker=dict(size=8, color='rgb(128, 128, 128)', line=dict(width=1, color='rgb(0, 0, 0)')
                                            ),
                                opacity=1)

        return point_peak



"""
Created on 21 de mar de 2018

@author: clebson
"""

from hydrocomp.files.fileRead import FileRead
import os
import pandas as pd
import numpy as np


class Ons(FileRead):

    source = "ONS"
    extension = "xlsx"

    def __init__(self, path_file=os.getcwd(), type_data='FLUVIOMÉTRICO', station=None, *args, **kwargs):
        super().__init__(path_file)
        self.type_data = type_data
        self.station = station
        if self.station is None:
            self.data = self.read(self.name)
        else:
            self.data = pd.DataFrame(self.read(self.name)[self.station])

    def list_files(self):
        return super().list_files()

    def read(self, name=None):
        if name is None:
            return super().read()
        else:
            self.name = name
            return self.__read_xls()

    def __read_xls(self):
        file_ons = os.path.join(self.path, self.name+'.'+Ons.extension)
        data_flow = pd.read_excel(file_ons, shettname='Total', header=0, skiprows=5, index_col=0)
        data_flow.drop(np.NaN, inplace=True)

        aux = []
        dic = {'jan': '1', 'fev': '2', 'mar': '3', 'abr': '4', 'mai': '5',
               'jun': '6', 'jul': '7', 'ago': '8', 'set': '9', 'out': '10',
               'nov': '11', 'dez': '12'
               }
        for i in data_flow.index:
            aux.append(i.replace(i[-8:-5], dic[i[-8:-5]]))

        data_flow.index = pd.to_datetime(aux, dayfirst=True)
        code_column = [i.split(' (')[0] for i in data_flow.axes[1]]
        data_flow.columns = code_column

        return data_flow.astype(float)



from .hydrogram_build import HydrogramBuild
import plotly.graph_objs as go
import pandas as pd
import colorlover as cl
import numpy as np


class HydrogramYear(HydrogramBuild):

    def __init__(self, data, title, threshold, width, height, size_text):
        self.data = data
        self.threshold = threshold
        super().__init__(width=width, height=height, size_text=size_text, title=title)

    def plot(self):
        group = self.group_by_year()
        number_of_lines = len(group.columns)
        ylrd = cl.scales['9']['div']['Spectral']
        ylrd = cl.interp(ylrd, number_of_lines)
        colors = dict(zip(group.columns, ylrd))
        trace = []
        for g in group:
            trace.append(go.Scatter(
                x=group[g].index,
                y=group[g].values,
                mode="lines",
                line=dict(color=colors[g]),
                name=g,)
            )

        colorbar_trace = go.Scatter(x=[None],
                                    y=[None],
                                    mode='markers',
                                    marker=dict(
                                        colorscale=ylrd,
                                        showscale=True,
                                        colorbar=dict(title='Ano'),
                                        cmin=group.columns[0],
                                        cmax=group.columns[-1],
                                    ),
                                    hoverinfo='none',
        )

        if self.threshold is not None:
            if type(self.threshold) is list:
                trace_threshold = []
                i = 1
                for t in self.threshold:
                    trace_threshold.append(self._plot_threshold(group, t, name="Limiar - {}".format(i)))
                    i += 1
            else:
                trace_threshold = [self._plot_threshold(group, self.threshold, name="Limiar")]
            data = trace + [colorbar_trace] + trace_threshold
        else:
            data = trace + [colorbar_trace]

        bandxaxis = go.layout.XAxis(
            title="Mês",
            tickformat="%b",
            linecolor='rgba(1,1,1,1)',
            gridcolor='rgba(1,1,1,1)'
        )

        bandyaxis = go.layout.YAxis(
            title="Vazão(m³/s)",
            showgrid=False,
        )

        layout = dict(
            title=dict(text=self.title,  x=0.5, xanchor='center', y=0.95, yanchor='top',
                       font=dict(family='Courier New, monospace', color='rgba(0,0,0,0)', size=self.size_text+6)),
            xaxis=bandxaxis,
            yaxis=bandyaxis,
            width=self.width, height=self.height,
            font=dict(family='Courier New, monospace', size=self.size_text, color='#7f7f7f'),
            showlegend=False, plot_bgcolor='rgba(0,0,0,0)', paper_bgcolor='rgba(0,0,0,0)')

        fig = dict(data=data, layout=layout)

        return fig, data

    def _plot_threshold(self, group, threshold, name):
        trace_threshold = go.Scatter(
            x=list(group[group.columns[3]].index),
            y=[threshold]*len(group),
            mode='lines+text',
            text=[name],
            textposition='top right',
            line=dict(color='rgb(0, 0, 0)',
                      width=1.5,
                      dash='dot')
        )

        return trace_threshold


    def group_by_year(self):
        list_year = []
        for key, data in self.data:
            aux = data.values.T
            index = data.index
            indexN = [pd.to_datetime('%s/%s/%s' % (i.month, i.day, 1998)) if i.month >= key.month else pd.to_datetime(
                '%s/%s/%s' % (i.month, i.day, 1999)) for i in index]
            serie = pd.Series(aux[0], index=indexN, name=key.year)
            list_year.append(serie)
        return pd.DataFrame(list_year).T


class NotStation(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "FitError: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")


class FitNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "FitNotExist: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")


class NotStatistic(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "NotStatistic: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")


class NotRva(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "NotRva: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")


class NotTypePandas(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "NotRva: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")


class ObjectErro(Exception):
    def __int__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ObjectErro: {}".format(self.message) + (" the line {}!".format(self.line) if self.line > 0 else "!")



import pandas as pd

from hydrocomp.comparasion.mae import MAE
from hydrocomp.comparasion.quantify_uncertainty import QuantifyUncertainty


class RMAE(QuantifyUncertainty):
    """
    Relative Mean Absolute Error - RMSE
    RMAE = MAE/mean(Qmax)
    """
    def __init__(self, reference, compared):
        super().__init__(reference, compared)

    def calculo_erro(self):
        mae = MAE(self.reference, self.compared).quantify()
        prob = list()
        rmae = list()
        for i in self.reference.index:
            prob.append(i)
            rmae.append(mae['MAE'][i] / self.reference[i])

        rmae_serie = pd.Series(rmae, index=prob, name='RMAE')
        return rmae_serie



from abc import ABCMeta, abstractmethod


class BootstrapBuild(object, metaclass=ABCMeta):

    def __init__(self, shape, location, scale, size):
        self.shape = shape
        self.location = location
        self.scale = scale
        self.size = size

    @abstractmethod
    def fit_resample(self):
        pass

    @abstractmethod
    def fits_resamples(self, quantity):
        pass

    @abstractmethod
    def magnitudes_resamples(self, quantity):
        pass



"""
Created on 21 de mar de 2018

@author: clebson
"""
from hydrocomp.files.fileRead import FileRead


class Semarh(FileRead):
    """
    class files read: Secretaria do Meio Ambiente e Recursos Hidreicos - SEMARH
    """
    source = "  SEMARH"
    extension = None

    def __init__(self, path_file, name):
        pass



import plotly.graph_objs as go
import numpy as np


class PermanenceCurve:

    def __init__(self, data, width=None, height=None, size_text=None, title=None):
        self.data = data
        self.width = width
        self.height = height
        self.size_text = size_text
        self.title = title

    def plot(self):
        bandxaxis = go.layout.XAxis(title="Probabilidade de excedência")
        bandyaxis = go.layout.YAxis(title="Vazão(m³/s)")


        layout = dict(title=self.title,
                      width=self.width, height=self.height,
                      xaxis=bandxaxis, yaxis=bandyaxis,
                      font=dict(family='Time New Roman', size=self.size_text, color='rgb(0,0,0)')
                      )

        data = list()
        data.append(self.data_plot(color='rgb(0,0,0)'))
        fig = dict(data=data, layout=layout)
        return fig, data

    def data_plot(self, color):
        data_y = np.sort(self.data.values)
        count = 0
        data_x = []
        for _ in data_y:
            count += 1
            data_x.append((1-(count/len(data_y)))*100)

        data = go.Scatter(x=data_x,
                          y=data_y,
                          line=dict(width=1, color=color),
                          opacity=1, connectgaps=False)
        return data


import pandas as pd

from hydrocomp.comparasion.quantify_uncertainty import QuantifyUncertainty


class MAE(QuantifyUncertainty):
    """
    Mean Absolute Error - RMSE
    MAE = [1/n * soma|(xi - Qmax)|]
    """
    def __init__(self, reference, compared):
        super().__init__(reference, compared)

    def calculo_erro(self):
        prob = list()
        mae = list()
        for i in self.reference.index:
            sum = 0
            prob.append(i)
            for j in self.compared[i].values:
                aux = abs(j - self.reference[i])
                sum += aux

            mae_value = ((1/len(self.compared))*sum)
            mae.append(mae_value)

        mae_serie = pd.Series(mae, index=prob, name='MAE')
        return mae_serie



import plotly.graph_objs as go


class Boxplot(object):

    def __init__(self, magn_resample=None, datas=None, name=None):
        self.magn_resample = magn_resample
        self.datas = datas
        self.name = name

    def plot(self):
        data = list()
        for i in self.magn_resample:
            data.append((go.Box(y=self.magn_resample[i].values,
                                name='%s Anos' % (i),
                                boxpoints='suspectedoutliers',
                                showlegend=False,
                                marker=dict(
                                    color='rgb(0,0,0)'
                                    )
                                )
                         ))

        layout = dict(title="Magnitudes",
                      showlegend=False,
                      width=1890, height=827,
                      font=dict(family='Time New Roman', size=34, color='rgb(0,0,0)')
                      )

        fig = dict(data=data, layout=layout)
        return fig, data

    def plot_comparasion(self):

        layout = dict(title="Magnitudes",
                      showlegend=False,
                      width=1890, height=827,
                      font=dict(family='Time New Roman', size=34, color='rgb(0,0,0)')
                      )

        fig = dict(data=self.datas, layout=layout)
        return fig, self.datas



import plotly.graph_objs as go

from abc import ABCMeta, abstractmethod


class HydrogramBuild(metaclass=ABCMeta):

    def __init__(self, width=None, height=None, title=None, size_text=None):
        self.width = width
        self.height = height
        self.title = title
        self.size_text = size_text

    def layout(self, bandxaxis, bandyaxis):
        layout = dict(
            title=dict(text=self.title, x=0.5, xanchor='center', y=0.95, yanchor='top',
                       font=dict(family='Courier New, monospace', size=self.size_text + 10)),
            xaxis=bandxaxis,
            yaxis=bandyaxis,
            width=self.width, height=self.height,
            font=dict(family='Courier New, monospace', size=self.size_text, color='rgb(0,0,0)'),
            showlegend=True, plot_bgcolor='#FFFFFF', paper_bgcolor='#FFFFFF')

        return layout

    @abstractmethod
    def plot(self):
        pass

    def _plot_one(self, data, station=None, color=None):
        data = go.Scatter(x=data.index,
                          y=data.T.values[0],
                          name=station,
                          line=dict(width=1, color=color),
                          opacity=1, connectgaps=False)
        return data



class FitError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "FitError: {}".format(self.message) + \
            (" the line {}!".format(self.line) if self.line > 0 else "!")


class FitNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "FitNotExist: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class AlphaError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "AlphaError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class ProbabilityError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ProbabilityError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class ValueError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ValueError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class ValueNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ValueNotExist: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class DataError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "DataError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class DataNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "DataNotExist: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class ParameterNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ParameterNotExist: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class ParameterError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ParameterError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class DistributionNotExist(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ParameterError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")


class DistributionError(Exception):
    def __init__(self, message, line=0):
        self.message = message
        self.line = line

    def __str__(self):
        return "ParameterError: {}".format(self.message) + \
               (" the line {}!".format(self.line) if self.line > 0 else "!")



import pandas as pd
import math
import scipy.stats as stat
import plotly as py
import plotly.figure_factory as FF

from hydrocomp.graphics.gantt import Gantt
from hydrocomp.graphics.genpareto import GenPareto
from hydrocomp.graphics.hydrogram_parcial import HydrogramParcial
from hydrocomp.graphics.boxplot import Boxplot
from hydrocomp.graphics.polar import Polar


class Parcial(object):
    distribution = 'GP'
    __percentil = 0.8
    dic_name = {'stationary': 'Percentil', 'events_by_year': 'Eventos por Ano', 'autocorrelation': 'Autocorrelação'}

    def __init__(self, obj, station, type_threshold, value_threshold, type_event, type_criterion, **kwargs):
        """
            Parameter:
                obj: Object Series;
                station: Point observation of date
                type_threshold: Tipo de calculo do limiar('stationary' or
                                                          'events_by_year')
                value_threshold: Valor do limiar:
                    Para type_threshold = 'stationary': percentil ou valor
                    Para type_threshold = 'events_by_year': quantidade média de
                                                            picos por ano
                type_criterion: Critério de Independência ('mean', 'median',
                                'xmin_bigger_dois_terco_x', 'xmin_bigger_qmin')
                type_event: Tipo do evento em estudo ('flood' or 'drought')
        """
        self.obj = obj
        self.data = self.obj.data
        self.station = station
        self.peaks = None
        self.threshold_criterion = None
        self.fit = None
        self.type_threshold = type_threshold
        self.type_criterion = type_criterion
        self.type_event = type_event
        self.value = value_threshold

        if self.type_criterion == 'median':
            self.__percentil = 0.65
        elif self.type_criterion == 'autocorrelation':
            self.duration = kwargs['duration']

        self.__threshold(self.value)
        if self.type_criterion is not None:
            self.name = '%s(%s) - %s' % (self.dic_name[self.type_threshold], self.value, self.type_criterion.title())
        else:
            self.name = '%s(%s)' % (self.dic_name[self.type_threshold], self.value)

        if self.peaks is None:
            self.event_peaks()

    def event_peaks(self):
        max_events = {'Date': list(), 'peaks': list(), 'Start': list(), 'End': list(),
                      'Duration': list()}

        events_criterion, self.threshold_criterion = self.__events_over_threshold()
        events_threshold = self.__events_over_threshold(self.threshold)[0]

        idx_before = events_threshold.index[0]
        low_limiar = False

        data = {'Date': list(), 'peaks': list()}
        data_min = {'Date': list(), 'peaks': list()}
        start = False
        for i in events_threshold.index:
            if not start:
                if events_threshold.loc[i]:
                    start = False
                else:
                    start = True
            if events_threshold.loc[i] and start:
                data['peaks'].append(self.data.loc[i, self.station])
                data['Date'].append(i)
                low_limiar = True
                data_min['peaks'].append(self.data.loc[i, self.station])
                data_min['Date'].append(i)
            else:
                if low_limiar:
                    data['peaks'].append(self.data.loc[idx_before, self.station])
                    data['Date'].append(i)
                    data['peaks'].append(self.data.loc[i, self.station])
                    data['Date'].append(i)
                    low_limiar = False
                    data_min['peaks'].append(self.data.loc[i, self.station])
                    data_min['Date'].append(i)
                # else:
                data, max_events, data_min = self.__criterion(data=data, max_events=max_events, data_min=data_min,
                                                              events_criterion=events_criterion.loc[i])
            idx_before = i

        self.peaks = pd.DataFrame(
            max_events, columns=['Duration', 'Start', 'End', 'peaks'],
            index=max_events['Date']
        )

        if (self.type_criterion == 'autocorrelation'
                and self.test_autocorrelation()[0]):
            self.duration += 1
            return self.event_peaks()
        if (self.type_threshold == 'events_by_year'
                and self.__test_threshold_events_by_year(self.peaks, self.value)):
            return self.event_peaks()
        return self.peaks

    def __peaks_type_event(self, lista):
        if self.type_event == "flood":
            return max(lista)
        elif self.type_event == "drought":
            return min(lista)
        else:
            return "Type Event Erro!"

    def __events_over_threshold(self, threshold=None):
        if threshold is None:
            if self.type_criterion == 'mean':
                threshold = self.data[self.station].mean()
            elif self.type_criterion == 'median':
                threshold = self.data[self.station].median()
            elif self.type_criterion is None:
                threshold = self.threshold

        if self.type_event == 'flood':
            events = self.data[self.station].isin(self.data.loc[self.data[
                                                                    self.station] > threshold, self.station])
            return events, threshold

        elif self.type_event == 'drought':
            events = self.data[self.station].isin(self.data.loc[self.data[
                                                                    self.station] < threshold, self.station])
            return events, threshold

        else:
            return 'Event error!'

    def __threshold(self, value):
        if value > 1 and self.type_threshold == 'events_by_year':
            self.threshold = self.data[self.station].quantile(self.__percentil)
        elif value > 1 and self.type_threshold == 'stationary':
            self.threshold = value
        else:
            self.threshold = self.data[self.station].quantile(value)
        return self.threshold

    def __test_threshold_events_by_year(self, peaks=None, value=None):
        n_year = self.obj.date_end.year - self.obj.date_start.year

        if len(peaks) + 1 < int(value * n_year):
            self.__percentil -= 0.005
            self.__threshold(self.__percentil)
            return True
        elif len(peaks) > (int(value * n_year) + 2):
            self.__percentil += 0.005
            self.__threshold(self.__percentil)
            return True
        return False

    def __criterion(self, **kwargs):
        if self.type_criterion == 'mean':
            data, max_events = self.__criterion_mean(data=kwargs['data'], max_events=kwargs['max_events'],
                                                     events_criterion=kwargs['events_criterion'])
            return data, max_events, kwargs['data_min']

        elif self.type_criterion == 'median':
            data, max_events = self.__criterion_median(data=kwargs['data'], max_events=kwargs['max_events'],
                                                       events_criterion=kwargs['events_criterion'])
            return data, max_events, kwargs['data_min']

        elif self.type_criterion == 'autocorrelation':
            data, max_events = self.__criterion_duration(data=kwargs['data'], max_events=kwargs['max_events'],
                                                         events_criterion=kwargs['events_criterion'])
            return data, max_events, kwargs['data_min']

        elif self.type_criterion == 'xmin_bigger_qmin':
            return self.__criterion_xmin_bigger_qmin(max_events=kwargs['max_events'], data_min=kwargs['data_min'],
                                                     data=kwargs['data'], events_criterion=kwargs['events_criterion'])

        elif self.type_criterion == 'xmin_bigger_dois_terco_x':
            return self.__criterion_xmin_bigger_dois_terco_x(data=kwargs['data'], max_events=kwargs['max_events'],
                                                             data_min=kwargs['data_min'],
                                                             events_criterion=kwargs['events_criterion'])

        elif self.type_criterion == 'duration_e_xmin':
            return self.__criterion_duration_and_xmin(data=kwargs['data'], max_events=kwargs['max_events'],
                                                      data_min=kwargs['data_min'],
                                                      events_criterion=kwargs['events_criterion'])

        elif self.type_criterion == 'duration':
            data, max_events = self.__criterion_duration(data=kwargs['data'], max_events=kwargs['max_events'],
                                                         events_criterion=kwargs['events_criterion'])
            return data, max_events, kwargs['data_min']

        elif self.type_criterion is None:
            data, max_events = self.__criterion_none(data=kwargs['data'], max_events=kwargs['max_events'])
            return data, max_events, kwargs['data_min']

    def __criterion_mean(self, data, max_events, events_criterion):
        if len(data['peaks']) > 0 and (not events_criterion):
            return self.__add_peaks(data, max_events)
        else:
            return data, max_events

    def __criterion_median(self, data, max_events, events_criterion):
        if len(data['peaks']) > 0 and (not events_criterion):
            return self.__add_peaks(data, max_events)
        else:
            return data, max_events

    def __criterion_none(self, data, max_events):
        if len(data['peaks']) > 0:
            return self.__add_peaks(data, max_events)
        else:
            return data, max_events

    def __criterion_duration(self, data, max_events, events_criterion):
        if not events_criterion:
            if len(data['peaks']) == 0:
                return data, max_events
            elif len(max_events['Date']) == 0:
                return self.__add_peaks(data, max_events)
            else:
                if self.__test_duration(data, max_events):
                    data, max_events = self.__troca_peaks(data, max_events)
                    return data, max_events
                return self.__add_peaks(data, max_events)
        else:
            return data, max_events

    def __criterion_xmin_bigger_qmin(self, data, max_events, data_min, events_criterion):
        if not events_criterion:
            if len(data['Date']) == 0:
                return data, max_events, data_min
            elif len(max_events['Date']) == 0:
                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
            elif len(data_min) == 0:
                return data, max_events, data_min
            else:
                if self.__test_xmin_bigger_q(data, max_events, data_min):
                    data, max_events = self.__troca_peaks(data, max_events)
                    data_min = {'Date': [], 'peaks': []}
                    return data, max_events, data_min
                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
        else:
            return data, max_events, data_min

    def __criterion_xmin_bigger_dois_terco_x(self, data, max_events, data_min, events_criterion):
        if not events_criterion:
            if len(data['Date']) == 0:
                return data, max_events, data_min
            elif len(max_events['Date']) == 0:
                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
            elif len(data_min) == 0:
                return data, max_events, data_min
            else:
                if self.__test_xmin_bigger_dois_terco_x(data, max_events, data_min):
                    data, max_events = self.__troca_peaks(data, max_events)
                    data_min = {'Date': [], 'peaks': []}
                    return data, max_events, data_min
                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
        else:
            return data, max_events, data_min

    def __criterion_duration_and_xmin(self, data, max_events, events_criterion, data_min):
        if not events_criterion:
            if len(data['Date']) == 0:
                return data, max_events, data_min
            elif len(max_events['Date']) == 0:
                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
            elif len(data_min) == 0:
                return data, max_events, data_min
            else:
                if self.__test_xmin_bigger_q(data, max_events, data_min) or \
                        self.__test_duration(data, max_events):
                    data, max_events = self.__troca_peaks(data, max_events)
                    data_min = {'Date': [], 'peaks': []}
                    return data, max_events, data_min

                data, max_events = self.__add_peaks(data, max_events)
                data_min = {'Date': [], 'peaks': []}
                return data, max_events, data_min
        else:
            return data, max_events, data_min

    def __test_duration(self, data, max_events):
        data_max = data['Date'][data['peaks'].index(
            self.__peaks_type_event(data['peaks']))]

        distancia_dias = data_max - max_events['Date'][-1]
        if distancia_dias.days < self.duration:
            return True
        return False

    def __test_xmin_bigger_q(self, data, max_events, data_min):
        q1 = max_events['peaks'][-1]
        data_q1 = max_events['Date'][-1]
        q2 = self.__peaks_type_event(data['peaks'])
        data_q2 = data['Date'][data['peaks'].index(q2)]
        df_data = pd.DataFrame(data_min['peaks'], index=data_min['Date'])
        df_data = df_data.loc[data_q1:data_q2]
        if self.type_event == "flood":
            menor = min(q1, q2)
            xmin = df_data.min().values

            if xmin > (3 / 4) * menor:
                return True
            return False
        elif self.type_event == "drought":
            maior = max(q1, q2)
            xmax = df_data.max().values

            if xmax < (3 / 4) * maior:
                return True
            return False
        else:
            return "Type Event Erro!"

    def __test_xmin_bigger_dois_terco_x(self, data, max_events, data_min):
        q1 = max_events['peaks'][-1]
        data_q1 = max_events['Date'][-1]
        q2 = self.__peaks_type_event(data['peaks'])
        data_q2 = data['Date'][data['peaks'].index(q2)]
        df_data = pd.DataFrame(data_min['peaks'], index=data_min['Date'])
        df_data = df_data.loc[data_q1:data_q2]
        if self.type_event == "flood":
            xmin = df_data.min().values
            if xmin > (2 / 3) * q1:
                return True
            return False
        elif self.type_event == "drought":
            xmax = df_data.max().values
            if xmax < (2 / 3) * q1:
                return True
            return False
        else:
            return "Type Event Erro!"

    def test_autocorrelation(self):
        try:
            n = len(self.peaks.peaks)
            serie = pd.Series(self.peaks.peaks, index=self.peaks.index)
            lag1 = serie.autocorr(lag=1)
            lag2 = serie.autocorr(lag=2)
            r11_n = (-1 + 1.645 * math.sqrt(n - 1 - 1)) / (n - 1)
            r12_n = (-1 - 1.645 * math.sqrt(n - 1 - 1)) / (n - 1)
            r21_n = (-1 + 1.645 * math.sqrt(n - 2 - 1)) / (n - 2)
            r22_n = (-1 - 1.645 * math.sqrt(n - 2 - 1)) / (n - 2)
            if r11_n > lag1 > r12_n and r21_n > lag2 > r22_n:
                return False, (r11_n, lag1, r12_n), (r21_n, lag2, r22_n)
            return True, (r11_n, lag1, r12_n), (r21_n, lag2, r22_n)
        except ValueError:
            self.event_peaks()
            self.test_autocorrelation()

    def __troca_peaks(self, data, max_events):

        if max_events['peaks'][-1] < self.__peaks_type_event(data['peaks']):
            max_events['peaks'][-1] = self.__peaks_type_event(data['peaks'])
            max_events['End'][-1] = data['Date'][-1]
            duration = max_events['End'][-1] - max_events['Start'][-1]
            max_events['Duration'][-1] = duration.days
            max_events['Date'][-1] = data['Date'][data['peaks'].index(self.__peaks_type_event(data['peaks']))]
            data = {'Date': list(), 'peaks': list()}
        else:
            max_events['End'][-1] = data['Date'][-1]
            duration = max_events['End'][-1] - max_events['Start'][-1]
            max_events['Duration'][-1] = duration.days
            data = {'Date': list(), 'peaks': list()}
        return data, max_events

    def __add_peaks(self, data, max_events):
        max_events['peaks'].append(self.__peaks_type_event(data['peaks']))
        max_events['Start'].append(data['Date'][0])
        max_events['End'].append(data['Date'][-1])
        duration = max_events['End'][-1] - max_events['Start'][-1]
        max_events['Duration'].append(duration.days)
        max_events['Date'].append(data['Date'][data['peaks'].index(self.__peaks_type_event(data['peaks']))])
        data = {'Date': list(), 'peaks': list()}
        return data, max_events

    def mvs(self):
        try:
            self.fit = stat.genpareto.fit(self.peaks['peaks'].values)
        except TypeError:
            self.event_peaks()
            self.fit = stat.genpareto.fit(self.peaks['peaks'].values)

        return self.fit

    def resample(self, quantity):
        try:
            n = len(self.peaks)
            df_resample = pd.DataFrame()
            for i in range(quantity):
                df = pd.DataFrame(self.peaks['peaks'].sample(n=n, replace=True).values, columns=['%s' % i])
                df_resample = df_resample.combine_first(df)
            return df_resample

        except AttributeError:
            self.event_peaks()
            return self.resample(quantity)

    def mvs_resample(self, quantity):
        dic = {'Parameter': list()}
        resample = self.resample(quantity)
        peaks = self.peaks.copy()
        for i in resample:
            self.peaks['peaks'] = resample[i].values
            dic['Parameter'].append(self.mvs())
        self.peaks = peaks
        return pd.DataFrame(dic)

    def magnitude(self, return_period):
        try:
            if type(return_period) is list:
                raise TypeError
            try:
                prob = 1 - (1 / return_period)
                mag = stat.genpareto.ppf(prob, self.fit[0], self.fit[1],
                                         self.fit[2])
                return mag

            except AttributeError:
                self.mvs()
                return self.magnitude(return_period)

        except TypeError:
            mag = self.__magnitudes(return_period)
            return mag

    def __magnitudes(self, return_periods, name=None):
        if name is None:
            name = self.name

        magns = list()
        for return_period in return_periods:
            mag = self.magnitude(return_period)

            magns.append(mag)

        return pd.Series(magns, index=return_periods, name=name)

    def magnitude_resample(self, quantity, return_period):
        magn = pd.DataFrame()
        para = self.mvs_resample(quantity)
        fit_origin = self.mvs()
        for i in para.index:
            self.fit = para['Parameter'][i]
            serie = self.__magnitudes(return_period, i)
            magn = magn.combine_first(serie.to_frame())

        self.fit = fit_origin
        magn_resample = magn.T
        return magn_resample

    def plot_distribution(self, title, type_function, save=False):
        try:
            genpareto = GenPareto(title, self.fit[0], self.fit[1], self.fit[2])
            data, fig = genpareto.plot(type_function)
            if save:
                aux_name = title.replace(' ', '_')
                py.image.save_as(fig, filename='gráficos/' + '%s.png' % aux_name)
            return data, fig
        except AttributeError:
            self.mvs()
            return self.plot_distribution(title, type_function)

    def hydrogram(self, title, save=False, width=None, height=None, size_text=16, color=None):
        hydrogram = HydrogramParcial(
            data=self.data, peaks=self.peaks,
            threshold=self.threshold, station=self.station,
            threshold_criterion=self.threshold_criterion, title=title, type_criterion=self.type_criterion, width=width,
            height=height, size_text=size_text, color=color)
        fig, data = hydrogram.plot()
        if save:
            aux_name = title.replace(' ', '_')
            py.image.save_as(fig, filename='gráficos/' + '%s.png' % aux_name)

        return fig, data

    def plot_boxplot_resample(self, magn_resample, name, save=False):

        data, fig = Boxplot(magn_resample=magn_resample, name=name).plot()
        if save:
            py.image.save_as(fig, filename='gráficos/boxplot_%s.png' % name)

        return fig, data

    def plot_spells(self, title, size_text=14):
        df_spells, df, month_start, month_end = Gantt.get_spells(data_peaks=self.peaks,
                                                                 month_water=[self.obj.month_num, self.obj.month_abr])

        df_spells = df_spells.sort_values('Task', ascending=False).reset_index(drop=True)

        colors = '#000000'
        fig = FF.create_gantt(df_spells, group_tasks=True, colors=colors, title=title, height=900, width=1200)

        #fig['data'][-1]['marker'].update(colorscale='Jet', colorbar=dict(title="m³/s", tickvals=df_spells.Complete,
        #                                                                 ticktext=df_spells.Name))

        fig['layout'].update(autosize=True)
        fig['layout']['xaxis'].update(title="Mês", range=[month_start, month_end], tickformat="%b")
        fig['layout']['yaxis'].update(title="Ano")
        fig['layout']['xaxis']['rangeselector'] = {}
        fig.layout.title = dict(text=title, x=0.5, xanchor='center', y=0.9, yanchor='top',
                                font=dict(family='Courier New, monospace', color='#7f7f7f', size=size_text + 6))
        fig.layout.font = dict(family='Courier New, monospace', size=size_text, color='#7f7f7f')
        fig.layout.plot_bgcolor = 'rgba(0,0,0,0)'
        return fig, df_spells

    def polar(self, save=False, width=900, height=900, size_text=14, title=None, color=None, name=None):
        if self.type_event == 'flood':
            if title is None:
                title = 'Maximum Partial'
            elif name is None:
                name = 'Maximum peaks'
        elif self.type_event == 'drought':
            if title is None:
                title = 'Minimum Partial'
            elif name is None:
                name = 'Minimum peaks'

        _polar = Polar(df_events=self.peaks)
        fig, data = _polar.plot(width=width, height=height, size_text=size_text, title=title, color=color, name=name)
        if save:
            py.image.save_as(fig, filename='graficos/polar_maximas_anuais.png')

        return fig, data



import plotly.graph_objs as go

from hydrocomp.graphics.hydrogram_build import HydrogramBuild


class HydrogramParcial(HydrogramBuild):

    def __init__(self, data, peaks, threshold, threshold_criterion=None, type_criterion=None, width=None,
                 height=None, size_text=None, title=None, station=None, color=None):
        super().__init__(width=width, height=height, size_text=size_text, title=title)
        self.data = data
        self.color = color
        self.peaks = peaks
        self.station = station
        self.threshold = threshold
        self.type_criterion = type_criterion
        self.threshold_criterion = threshold_criterion

    def plot(self):
        bandxaxis = go.layout.XAxis(title="Date")
        bandyaxis = go.layout.YAxis(title="Flow(m³/s)")

        try:
            if self.threshold_criterion is None:
                raise AttributeError

            layout = self.layout(bandxaxis=bandxaxis, bandyaxis=bandyaxis)

            data = []

            for i in self.data:
                try:
                    color = self.color[i]
                except:
                    color = None

            data.append(self._plot_one(self.data, station=self.station, color=color))
            data.append(self._plot_threshold())
            if self.type_criterion is not None:
                data.append(self._plot_threshold_criterion())
            data += self._plot_event_peaks()

            fig = dict(data=data, layout=layout)
            return fig, data

        except AttributeError:

            layout = self.layout(bandxaxis=bandxaxis, bandyaxis=bandyaxis)

            data = []
            for i in self.data:
                print(i)
                try:
                    color = self.color[i]
                except:
                    color = None

            data.append(self._plot_one(self.data, station=self.station, color=color))
            data.append(self._plot_threshold())
            data += self._plot_event_peaks()

            fig = dict(data=data, layout=layout)
            return fig, data


    def _plot_event_peaks(self):
        point_start = go.Scatter(
            x=list(self.peaks.Start),
            y=self.data.loc[self.peaks.Start].T.values[0],
            name="Start of events",
            mode='markers',
            marker=dict(color='rgb(0, 0, 0)',
                        symbol='circle-dot',
                        size=6),
            opacity=1)

        point_end = go.Scatter(
            x=list(self.peaks.End),
            y=self.data.loc[self.peaks.End].T.values[0],
            name="Ends of events",
            mode='markers',
            marker=dict(color='rgb(0, 0, 0)',
                        size=6,
                        symbol="x-dot"),
            opacity=1)

        point_vazao = go.Scatter(
            x=self.peaks['peaks'].index,
            y=self.peaks['peaks'].values,
            name="Peaks",
            mode='markers',
            marker=dict(size=8,
                        color='rgb(128, 128, 128)',
                        line=dict(width=1,
                                  color='rgb(0, 0, 0)'),),
            opacity=1)

        return [point_start, point_end, point_vazao]

    def _plot_threshold(self):
        trace_threshold = go.Scatter(
            x=self.data.index,
            y=[self.threshold]*len(self.data),
            name="Threshold",
            line=dict(color='rgb(128, 128, 128)',
                      width=1.5,
                      dash='dot')
        )

        return trace_threshold

    def _plot_threshold_criterion(self):
        trace_threshold_criterion = go.Scatter(
            x=self.data.index,
            y=[self.threshold_criterion]*len(self.data),
            name=self.type_criterion,
            line=dict(color='rgb(128, 128, 128)',
                      width=1.5,
                      dash='dash')
        )
        return trace_threshold_criterion



from hydrocomp.iha.exceptions import *
import pandas as pd
import calendar as cal
import numpy as np
from hydrocomp.series.flow import Flow


class IHA:
    group = {
        'group1': 'magnitude',
        'group2': 'magnitude_and_duration',
        'group3': 'timing_extreme',
        'group4': 'frequency_and_duration',
        'group5': 'rate_and_frequency'
    }

    def __init__(self, data, month_water=None, station=None, status=None, date_start=None, date_end=None,
                 statistic=None, central_metric=None, variation_metric=None, type_threshold=None, type_criterion=None,
                 threshold_high=None, threshold_low=None, **kwargs):
        """
        :param data: pandas Series
        :param month_water: initial month water (int: referent the month, ex.: 1 for Jan, 2 for Fev)
        :param station: station of data
        :param status: 'pre' or 'pos'
        :param date_start: Date of start application ('dd/mm/aaaa')
        :param date_end: Date of end application ('dd/mm/aaaa')
        :param statistic: 'non-parametric or parametric'
        :param central_metric: 'mean or median'
        :param variation_metric: 'str' or 'cv'
        """
        self.source = kwargs['source']
        self.flow = Flow(data, source=self.source, station=station)
        self.station = self.get_station(station)
        self.status = status
        self.month_start = self.get_month_start(month_water)
        self.date_start = pd.to_datetime(date_start, dayfirst=True)
        self.date_end = pd.to_datetime(date_end, dayfirst=True)
        self.statistic = statistic
        self.central_metric = central_metric
        self.variation_metric = variation_metric
        self.type_threshold = type_threshold
        self.type_criterion = type_criterion
        self.threshold_high = threshold_high
        self.threshold_low = threshold_low
        self.kwargs = kwargs

    # <editor-fold desc="Range of Variability Approach"
    def rva(self, iha_other, group_iha=None, boundaries=17):

        def rva_very(rva):
            for i in rva:
                for j in rva[i].index:
                    if rva[i][j] < -1:
                        rva.iat[i, j] = -1
            return rva

        if group_iha is None:
            for i in self.group:
                return self.rva(iha_other=iha_other, group_iha=i)
        else:
            data_group, _ = eval('self.' + self.group[group_iha])()
            data_group_other, _ = eval('iha_other.' + iha_other.group[group_iha])()
            if iha_other.status != self.status:
                if iha_other.status is 'pos':
                    line = self.rva_line(data_group, boundaries=boundaries)
                    group_iha = self.rva_frequency(line=line, data_group=data_group)
                    group_iha_other = self.rva_frequency(line=line, data_group=data_group_other)
                    rva = (group_iha_other - group_iha) / group_iha
                    return rva_very(rva), line

                elif iha_other.status is 'pre':
                    line = iha_other.rva_line(data_group)
                    group_iha = self.rva_frequency(line=line, data_group=data_group)
                    group_iha_other = self.rva_frequency(line=line, data_group=data_group_other)
                    rva = (group_iha - group_iha_other) / group_iha_other
                    return rva_very(rva), line
                else:
                    raise ObjectErro('Status Erro')
            else:
                raise ObjectErro("Status equals")

    # </editor-fold>

    # <editor-fold desc="Calculation of statistical metrics">
    @staticmethod
    def metric_stats(group, central_metric, variation_metric):
        if central_metric == 'mean':
            mean = pd.DataFrame(group.mean(), columns=['Means'])
        elif central_metric == 'median':
            mean = pd.DataFrame(group.median(), columns=['Means'])
        else:
            raise NotStatistic('Not exist statistic {}: use {} or {}'.format(central_metric, 'mean', 'median'), line=40)
        if variation_metric == 'cv':
            cv = pd.DataFrame(group.std() / group.mean(), columns=['Coeff. of Var.'])
        elif variation_metric == 'std':
            cv = pd.DataFrame(group.std(), columns=['Coeff. of Var.'])
        else:
            raise NotStatistic('Not exist statistic {}: use {} or {}'.format(variation_metric, 'cv', 'std'), line=46)

        stats = mean.combine_first(cv)
        return stats

    # </editor-fold>

    # <editor-fold desc="Check type rate"
    @staticmethod
    def check_rate(value1, value2, type_rate):
        if type_rate == 'Rise rate':
            return value1 < value2
        elif type_rate == 'Fall rate':
            return value1 > value2

    # </editor-fold>

    # <editor-fold desc="Range of Variability Approach Frequency">
    @staticmethod
    def rva_frequency(data_group, line):
        count = pd.DataFrame(columns=['Lower', 'Median', 'Upper'])
        upper_line, lower_line = line['upper_line'], line['lower_line']
        for i in data_group:
            if upper_line[i] == 0 and lower_line[i] == 0:
                count.at[i, 'Lower'] = 0
                count.at[i, 'Upper'] = 0
                count.at[i, 'Median'] = 0
            else:
                boolean_lower = pd.DataFrame(data_group[i].isin(data_group.loc[data_group[i] <= lower_line[i], i]))
                boolean_upper = pd.DataFrame(data_group[i].isin(data_group.loc[data_group[i] >= upper_line[i], i]))
                boolean_median = pd.DataFrame(data_group[i].isin(data_group.loc[
                                                                     boolean_lower[i] == boolean_upper[i], i]))

                count.at[i, 'Lower'] = boolean_lower[i].loc[boolean_lower[i] == True].count()
                count.at[i, 'Upper'] = boolean_upper[i].loc[boolean_upper[i] == True].count()
                count.at[i, 'Median'] = boolean_median[i].loc[boolean_median[i] == True].count()
        return count

    # </editor-fold>

    # <editor-fold desc="Range of Variability Approach Line">
    def rva_line(self, data_group, boundaries):
        if (type(data_group) == type(pd.DataFrame())) or (type(data_group) == type(pd.Series())):
            if self.status == 'pre':
                line = pd.DataFrame(columns=['lower_line', 'upper_line', 'median_line'])

                if self.statistic == 'non-parametric':
                    for i in data_group:
                        line.at[i, 'lower_line'] = data_group[i].quantile((50 - boundaries) / 100)
                        line.at[i, 'upper_line'] = data_group[i].quantile((50 + boundaries) / 100)
                        line.at[i, 'median_line'] = data_group[i].median()
                    return line
                elif self.statistic == 'parametric':
                    for i in data_group:
                        line.at[i, 'lower_line'] = data_group[i].mean() - data_group[i].std()
                        line.at[i, 'lower_line'] = data_group[i].mean() + data_group[i].std()
                        line.at[i, 'lower_line'] = data_group[i].median()
                    return line
                else:
                    raise NotStatistic('Not exist statistic {}: use {} or {}'.format(
                        self.statistic, 'non-parametric', 'parametric'), line=91)
            else:
                raise NotRva('Use RVA in data pre-impact', line=93)
        else:
            raise NotTypePandas('Not use type data {}: Use {} or {}'.format(type(data_group), type(pd.DataFrame()),
                                                                            type(pd.Series())), line=96)

    # </editor-fold>

    # <editor-fold desc="Return Station">
    def get_station(self, station):
        if len(self.flow.data.columns.values) != 1:
            if station is None:
                raise NotStation("Station requirement")
            else:
                if station in self.flow.data.columns.values:
                    return station
                else:
                    raise NotStation("Not station")
        else:
            get_station = self.flow.data.columns.values[0]
            self.flow.data = pd.DataFrame(self.flow.data[get_station])

            return get_station

    # </editor-fold>

    # <editor-fold desc= "Return Month Water">
    def get_month_start(self, month_water=None):
        """
        :param month_water:
        :return self.month_water:
        """
        if month_water is None:
            return self.flow.month_start_year_hydrologic()
        else:
            self.flow.month_abr = 'AS-%s' % cal.month_abbr[month_water].upper()
            self.flow.month_num = month_water
            return self.flow.month_num, self.flow.month_abr

    # </editor-fold>

    # <editor-fold desc="Group 1: Magnitude of monthly water conditions">
    def magnitude(self):
        years = self.flow.data.groupby(pd.Grouper(freq=self.month_start[1]))
        data = pd.DataFrame()
        for year in years:
            aux = year[1].groupby(pd.Grouper(freq='M')).mean()
            df = pd.DataFrame({year[0].year: {
                i.month: aux[self.station][i] for i in aux[self.station].index}})
            data = data.combine_first(df)
        mean_months = data.T

        return mean_months, self.metric_stats(mean_months, central_metric=self.central_metric,
                                              variation_metric=self.variation_metric)

    # </editor-fold>

    # <editor-fold desc="Group 2: Magnitude and Duration of annual extreme water conditions">
    def magnitude_and_duration(self):
        aver_data = pd.DataFrame()
        for i in [1, 3, 7, 30, 90]:
            ave_max = self.flow.data.rolling(window=i).mean().groupby(pd.Grouper(freq=self.month_start[1])).max()
            ave_min = self.flow.data.rolling(window=i).mean().groupby(pd.Grouper(freq=self.month_start[1])).min()
            years = ave_max.index.year
            df1 = pd.DataFrame(pd.Series(data=ave_min[self.station].values, name='%s-day minimum' % i, index=years))
            df2 = pd.DataFrame(pd.Series(data=ave_max[self.station].values, name='%s-day maximum' % i, index=years))
            aver_data = aver_data.combine_first(df1)
            aver_data = aver_data.combine_first(df2)
            if i == 7:
                mean_year = self.flow.data[self.station].groupby(pd.Grouper(freq=self.month_start[1])).mean().values
                base_flow = pd.DataFrame(pd.Series(data=ave_min[self.station].values / mean_year,
                                                   name='Base flow index', index=years))
                aver_data = aver_data.combine_first(base_flow)

        dic_zero = {i[0].year: i[1].loc[i[1][self.station].values == 0].sum().values[0]
                    for i in self.flow.data.groupby(pd.Grouper(freq=self.month_start[1]))}
        serie_dict_zero = pd.Series(data=dic_zero, name='Number of zero days')
        serie_dict_zero.loc[serie_dict_zero.isnull()].value = 0
        magn_and_duration = aver_data.combine_first(pd.DataFrame(serie_dict_zero))

        return magn_and_duration, self.metric_stats(magn_and_duration, central_metric=self.central_metric,
                                                    variation_metric=self.variation_metric)

    # </editor-fold>

    # <editor-fold desc="Group 3: Timing of annual extreme water conditions">

    def timing_extreme(self):

        day_julian_max = self.flow.data[self.station].groupby(
            pd.Grouper(freq=self.month_start[1])).idxmax().dropna(axis=0)
        day_julian_min = self.flow.data[self.station].groupby(
            pd.Grouper(freq=self.month_start[1])).idxmin().dropna(axis=0)

        df_day_julian_max = pd.DataFrame(list(map(int, pd.DatetimeIndex(day_julian_max.values).strftime("%j"))),
                                         index=day_julian_max.index.year,
                                         columns=["Date of maximum"])

        df_day_julian_min = pd.DataFrame(list(map(int, pd.DatetimeIndex(day_julian_min.values).strftime("%j"))),
                                         index=day_julian_min.index.year,
                                         columns=["Date of minimum"])

        # combine the dfs of days julian
        timing_extreme = df_day_julian_max.combine_first(df_day_julian_min)
        return timing_extreme, self.metric_stats(timing_extreme, central_metric=self.central_metric,
                                                 variation_metric=self.variation_metric)

    # </editor-fold>

    # <editor-fold desc="Group 4: Frequency and duration of high and low pulses">
    def frequency_and_duration(self):

        def aux_frequency_and_duration(events):
            name = {'flood': 'High', 'drought': 'Low'}
            type_event = events.type_event

            group = pd.DataFrame(
                index=pd.date_range(events.obj.date_start, events.obj.date_end, freq=self.month_start[1]),
                columns=["{} pulse duration".format(name[type_event]),
                         '{} pulse count'.format(name[type_event])])

            if len(events.peaks) != 0:
                duration_pulse = pd.DataFrame(events.peaks.groupby(
                    pd.Grouper(freq=self.month_start[1])).Duration.mean()).rename(
                    columns={"Duration": '{} pulse duration'.format(name[type_event])})

                pulse = pd.DataFrame(events.peaks.resample(self.month_start[1]).count().Duration).rename(
                    columns={"Duration": '{} pulse count'.format(name[type_event])})

                group = group.combine_first(duration_pulse)
                group = group.combine_first(pulse)

            group = group.fillna(value={'{} pulse count'.format(name[type_event]): 0})
            threshold = pd.DataFrame(pd.Series(events.threshold, name="{} Pulse Threshold".format(name[type_event])))
            return group, threshold

        events_high = self.flow.parcial(type_threshold=self.type_threshold, type_event="flood",
                                        type_criterion=self.type_criterion, value_threshold=self.threshold_high)

        frequency_and_duration_high, threshold_high_mag = aux_frequency_and_duration(events_high)

        events_low = self.flow.parcial(type_event='drought', type_threshold=self.type_threshold,
                                       type_criterion=self.type_criterion, value_threshold=self.threshold_low)
        frequency_and_duration_low, threshold_low_mag = aux_frequency_and_duration(events_low)

        frequency_and_duration = frequency_and_duration_high.combine_first(frequency_and_duration_low)
        metric_stats = self.metric_stats(frequency_and_duration, central_metric=self.central_metric,
                                         variation_metric=self.variation_metric)

        return frequency_and_duration, metric_stats, events_high, events_low

    # </editor-fold>

    # <editor-fold desc="Group 5: Rate and frequency of water condition changes">
    def rate_and_frequency(self):
        """

        :return:
        """
        data_water = self.flow.data.groupby(pd.Grouper(freq=self.month_start[1]))
        rate_df = pd.DataFrame(columns=['Rise rate', 'Fall rate', 'Number of reversals'])
        boo = False
        mean = 0
        for type_rate in ['Rise rate', 'Fall rate']:
            for key, series in data_water:
                d1 = None
                values = []
                cont = 0
                for i in series.index:
                    if d1 is not None:
                        if self.check_rate(self.flow.data.loc[d1, self.station], self.flow.data.loc[i, self.station],
                                           type_rate):
                            boo = True
                            values.append(self.flow.data.loc[i, self.station] - self.flow.data.loc[d1, self.station])
                        else:
                            if boo:
                                mean = np.mean(values)
                                cont += 1
                                boo = False
                    d1 = i
                if boo:
                    mean = np.mean(values)
                    boo = False

                rate_df.at[key.year, type_rate] = mean
                if rate_df['Number of reversals'][key.year] > 0:
                    rate_df.at[key.year, 'Number of reversals'] = cont + rate_df['Number of reversals'][key.year]
                else:
                    rate_df.at[key.year, 'Number of reversals'] = cont
        return rate_df, self.metric_stats(rate_df, central_metric=self.central_metric,
                                          variation_metric=self.variation_metric)
    # </editor-fold>



import pandas as pd

from abc import ABCMeta, abstractmethod


class QuantifyUncertainty(object, metaclass=ABCMeta):

    def __init__(self, reference, compared):
        """Dados de entrada:
            reference: <pd.DataFrame> com as magnitudes estimadas pela distribuiçao
            de referência.
            compared: <dict> contendo os <pd.DataFrame> com as magnitudes estimadas
            pelas distribuições a serem comparadas.
        """
        self.reference = reference
        self.compared = compared

    @abstractmethod
    def calculo_erro(self):
        pass

    def quantify(self):
        df_comp = pd.DataFrame()
        serie_comp = self.calculo_erro()

        df_comp = serie_comp.to_frame()

        return df_comp



import numpy as np

from hydrocomp import statistic as e
from hydrocomp.statistic.stats_build import StatsBuild
from scipy.stats import genpareto
from lmoments3.distr import gpa


class Gpa(StatsBuild):

    name = 'GPA'
    estimator = None
    parameter = {'shape': None, 'loc': None, 'scale': None}

    def __init__(self, data=None,  shape=None, loc=None, scale=None):
        self.shape = shape
        self.loc = loc
        self.scale = scale
        self.parameter['shape'] = self.shape
        self.parameter['loc'] = self.loc
        self.parameter['scale'] = self.scale
        super().__init__(data, shape, loc, scale)
        self.dist = genpareto(c=self.shape, loc=self.loc, scale=self.scale)
    
    def mml(self):
        if self.data is None:
            raise e.DataNotExist("Data not's None", 25)
        mml = gpa.lmom_fit(self.data)
        self.estimador = 'MML'
        self.shape = mml['c']
        self.loc = mml['loc']
        self.scale = mml['scale']
        self.dist = genpareto(c=self.shape, loc=self.loc, scale=self.scale)

        return self.shape, self.loc, self.scale

    def mvs(self):
        if self.data is None:
            raise e.DataNotExist("Data not's None", 35)
        mvs = genpareto.fit(data=self.data)
        self.estimador = 'MVS'
        self.shape = mvs[0]
        self.loc = mvs[1]
        self.scale = mvs[2]
        self.dist = genpareto(c=self.shape, loc=self.loc, scale=self.scale)

        return self.shape, self.loc, self.scale



import os

from hydrocomp.series.series_build import SeriesBuild
from hydrocomp.graphics.hydrogram_clean import HydrogramClean


class Height(SeriesBuild):

    type_data = 'COTA'

    def __init__(self, data=None, path_file=os.getcwd(), source=None, *args, **kwargs):
        super().__init__(data, path_file, source, type_data=self.type_data, *args, **kwargs)

    def month_start_year_hydrologic(self):
        pass

    def plot_hydrogram(self, title, save=False, width=None, height=None, size_text=None):
        if self.station is None:
            hydrogram = HydrogramClean(self.data, width=width, height=height, size_text=size_text,
                                       title=title, y_title='Cota (m)', x_title='Data')
            fig, data = hydrogram.plot()
        else:
            hydrogram = HydrogramClean(self.data[self.station], width=width, height=height,
                                       size_text=size_text, title=title, y_title='Cota (m)', x_title='Data')
            fig, data = hydrogram.plot()
        return fig, data



import pandas as pd
import plotly.graph_objs as go
from hydrocomp.graphics.hydrogram_build import HydrogramBuild


class HydrogramClean(HydrogramBuild):

    def __init__(self, data, threshold=None, width=None, height=None, size_text=None, title=None, y_title=None,
                 x_title=None, color=None):
        super().__init__(width=width, height=height, size_text=size_text, title=title)
        self.threshold = threshold
        self.data = pd.DataFrame(data)
        self.y_title = y_title
        self.x_title = x_title
        self.color = color

    def plot(self):
        bandxaxis = go.layout.XAxis(title=self.x_title)
        bandyaxis = go.layout.YAxis(title=self.y_title)

        if len(self.data.columns.values) == 1:

            layout = dict(
                title=dict(text=self.title, x=0.5, xanchor='center', y=0.95, yanchor='top',
                           font=dict(family='Courier New, monospace', size=self.size_text + 10)),
                xaxis=bandxaxis,
                yaxis=bandyaxis,
                width=self.width, height=self.height,
                font=dict(family='Courier New, monospace', size=self.size_text, color='rgb(0,0,0)'),
                showlegend=True, plot_bgcolor='#FFFFFF', paper_bgcolor='#FFFFFF')

            data = list()
            data.append(self._plot_one(self.data, self.title, color='rgb(0,0,0)'))

            if self.threshold is not None:
                if type(self.threshold) is list:
                    trace_threshold = []
                    i = 1
                    for t in self.threshold:
                        trace_threshold.append(self._plot_threshold(self.data, t, name="Limiar - {}".format(i)))
                        i += 1
                else:
                    trace_threshold = [self._plot_threshold(self.data, self.threshold, name="Limiar")]
                data = data + trace_threshold
            else:
                pass

            fig = dict(data=data, layout=layout)
            return fig, data

        else:

            data, buttons = self._plot_multi()

            if self.threshold is not None:
                if type(self.threshold) is list:
                    trace_threshold = []
                    i = 1
                    for t in self.threshold:
                        trace_threshold.append(self._plot_threshold(self.data, t, name="Threshold - {}".format(i)))
                        i += 1
                else:
                    trace_threshold = [self._plot_threshold(self.data, self.threshold, name="Threshold")]
                data = data + trace_threshold
            else:
                pass

            menus = False
            if menus:
                update_menus = go.layout.Updatemenu(active=0, buttons=list(buttons))
            else:
                update_menus = None

            layout = dict(
                title=dict(text=self.title, x=0.5, xanchor='center', y=0.95, yanchor='top',
                           font=dict(family='Courier New, monospace', size=self.size_text + 10)),
                xaxis=bandxaxis,
                yaxis=bandyaxis,
                width=self.width, height=self.height,
                font=dict(family='Courier New, monospace', size=self.size_text, color='rgb(0,0,0)'),
                showlegend=True, plot_bgcolor='#FFFFFF', paper_bgcolor='#FFFFFF', updatemenus=update_menus)
            fig = dict(data=data, layout=layout)
            return fig, data

    def _plot_threshold(self, group, threshold, name):
        trace_threshold = go.Scatter(
            x=list(group[group.columns[0]].index),
            y=[threshold]*len(group),
            mode='lines',
            name=name,
            textposition='top right',
            line=dict(color='rgb(0, 0, 0)',
                      width=1.5,
                      dash='dot')
        )

        return trace_threshold

    def _plot_multi(self):
        data = []
        buttons = [dict(label="All",
                        method='update',
                        args=[{"visible": [True] * len(self.data.columns)},
                              {"title": "Hidrograma"}]
                        )]
        aux = 0
        visible = [False] * len(self.data.columns)
        for i in self.data:
            visible[aux] = True
            try:
                color = self.color[i]
            except:
                color = None

            data.append(self._plot_one(pd.DataFrame(self.data[i]), station=i, color=color))

            buttons.append(
                dict(label=i,
                     method='update',
                     args=[{"visible": visible},
                           {"title": "Hidrograma: {}".format(i)}]
                     )
            )

            visible = [False] * len(self.data.columns)
            aux += 1
        return data, buttons



"""
Created on 21 de mar de 2018

@author: clebson
"""

import os
import calendar as ca

import numpy as np
import pandas as pd
from hydrocomp.files.fileRead import FileRead
from api_ana.serie_temporal import SerieTemporal


class Ana(FileRead):
    """
    class files read: Agência Nacinal de Águas - ANA
    """
    typesData = {'FLUVIOMÉTRICO': ['Vazao{:02}', 'vazoes', '3'],
                 'PLUVIOMÉTRICO': ['Chuva{:02}', 'chuvas', '2'],
                 'COTA': ['Cota{:02}', 'cotas', '1']}
    source = "ANA"
    extension = "txt"

    def __init__(self, path_file=None, station=None, type_data='FLUVIOMÉTRICO', consistence='1', date_start='',
                 date_end='', *args, **kwargs):
        super().__init__(path_file=path_file, station=station, *args, **kwargs)
        self.consistence = consistence
        self.date_start = date_start
        self.date_end = date_end
        self.type_data = type_data.upper()
        self.data = self.read(self.name)
        try:
            self.mean = kwargs["mean"]
        except KeyError:
            self.mean = None

    def list_files(self):
        return super().list_files()

    def read(self, name=None):
        if self.api:
            if name is None:
                self.name = self.path
                return super().read()
            else:
                self.name = name
                data = self.__excludes_duplicates(self.hidro_serie_historica())
        else:
            if name is None or name is not list:
                self.name = self.list_files()
                return super().read()
            else:
                self.name = name
                data = self.__excludes_duplicates(self.__readTxt())
        return data

    def __lines(self):
        list_lines = []
        with open(os.path.join(self.path, self.name + '.' + Ana.extension),
                  encoding="Latin-1") as file:
            l = 0
            for line in file.readlines():
                if line.split(";")[0] == "EstacaoCodigo":
                    l = 1
                    list_lines.append(line.split(";"))
                elif l == 1:
                    list_lines.append(line.split(";"))

        return list_lines

    def __multIndex(self, date, days, consistence):
        if date.day == 1:
            n_days = days
        else:
            n_days = days - date.day
        list_date = pd.date_range(date, periods=n_days, freq="D")
        list_cons = [int(consistence)] * n_days
        index_multi = list(zip(*[list_date, list_cons]))
        return pd.MultiIndex.from_tuples(index_multi, names=["Date", "Consistence"])

    def __readTxt(self):
        list_lines = self.__lines()
        data_flow = list()
        count = 0
        for line in list_lines:
            count += 1
            if count == 1:
                idx_code = line.index("EstacaoCodigo")
                start_flow = line.index(Ana.typesData[self.type_data][0].format(1))
                idx_date = line.index("Data")
                idx_cons = line.index("NivelConsistencia")
            elif count >= 2:
                code = line[idx_code]
                date = pd.to_datetime(line[idx_date], dayfirst=True)
                days = ca.monthrange(date.year, date.month)[1]
                consistence = line[idx_cons]
                index = self.__multIndex(date, days, consistence)
                idx_flow = [i for i in range(start_flow, start_flow + days)]
                list_flow = [np.NaN if line[i] == "" else float(
                    line[i].replace(",", ".")) for i in idx_flow]
                data_flow.append(
                    pd.Series(list_flow, index=index, name="{}_{}".format(code, self.type_data[:3])))
        data_flow = pd.DataFrame(pd.concat(data_flow))
        return data_flow

    def hidro_serie_historica(self):
        serie_temporal = SerieTemporal()
        data = serie_temporal.get(codEstacao=self.name, dataInicio=self.date_start, dataFim=self.date_end,
                                  tipoDados=Ana.typesData[self.type_data][2], nivelConsistencia=self.consistence)
        return data

    def __excludes_duplicates(self, data):
        if len(data) > 0:
            if self.consistence == '1':  # bruto_e_consistido
                ordem = data.copy(deep=True)
                eh_duplicata = ordem.reset_index(level=1, drop=True).index.duplicated(keep='last')
                saida = data[~eh_duplicata]
            elif self.consistence == '2':  # somente consistidos
                try:
                    saida = data.iloc[data.index.isin([self.consistence], level=1)]
                except KeyError:
                    return
        else:
            return data
        return saida.reset_index(level=1, drop=True).groupby(pd.Grouper(freq='D')).mean()



import plotly.graph_objs as go
import pandas as pd


class Polar(object):

    def __init__(self, df_events):
        self.df_events = df_events

    def year_polar(self):
        date_julian = list(map(int, pd.DatetimeIndex(self.df_events.index.values).strftime("%j")))
        date_julian_polar = [(i/366)*360 for i in date_julian]
        df_events_julian = self.df_events.copy()
        df_events_julian['DateJulian'] = date_julian
        df_events_julian['DateJulianPolar'] = date_julian_polar
        return df_events_julian

    def plot(self, width=None, height=None, size_text=None, title=None, color=None, name=None):
        dicMes = ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set",
                  "Out", "Nov", "Dez"]
        df_polar = self.year_polar()
        position = [0, 31*0.9863, 59*0.9863, 90*0.9863, 120*0.9863, 151*0.9863, 181*0.9863, 212*0.9863, 243*0.9863,
                    273*0.9863, 303*0.9863, 334*0.9863]

        trace = go.Scatterpolar(
            r=df_polar.peaks.values,  # Vazao
            theta=df_polar.DateJulianPolar.values,  # Data
            mode='markers',
            marker=dict(
                color=color,
                size=10,
                line=dict(
                    color='white'
                ),
                opacity=0.7),
            text=df_polar.index.date,
            thetaunit='degrees',
            name=name
        )

        data = [trace]
        angularX = go.layout.AngularAxis(
            showticklabels=False,
        )

        layout = dict(
            angularaxis=angularX,
            title=dict(text=title, x=0.5, xanchor='center', y=0.95, yanchor='top',
                       font=dict(family='Courier New, monospace', color='#7f7f7f', size=size_text + 6)),
            width=width, height=height,
            font=dict(family='Courier New, monospace', size=size_text, color='#7f7f7f'),
            showlegend=True, plot_bgcolor='#FFFFFF', paper_bgcolor='#FFFFFF',
            polar=dict(
                radialaxis=dict(showticklabels=True, gridcolor="#000000"),
                angularaxis=dict(showticklabels=True, ticks='', tickvals=position, ticktext=dicMes, rotation=90,
                                 direction="clockwise", gridcolor="#000000", tickcolor="#000000"),
                bgcolor='#FFFFFF',
            )
        )

        fig = dict(data=data, layout=layout)
        return fig, data



import pandas as pd

from hydrocomp.comparasion.quantify_uncertainty import QuantifyUncertainty


class RMSE(QuantifyUncertainty):
    """
    Root Mean Square Error - RMSE
    RMSE = [1/n * soma(xi - Qmax)²]^1/2
    """
    def __init__(self, reference, compared):
        super().__init__(reference, compared)

    def calculo_erro(self):
        prob = list()
        rmse = list()
        for i in self.reference.index:
            sum = 0
            prob.append(i)
            for j in self.compared[i].values:
                aux = (j - self.reference[i]) ** 2
                sum += aux

            rmse_value = (1/len(self.compared)*sum) ** 0.5
            rmse.append(rmse_value)

        rmse_serie = pd.Series(rmse, index=prob, name='RMSE')
        return rmse_serie



"""
Created on 21 de mar de 2018

@author: clebson
"""

import os
from abc import ABCMeta, abstractmethod


class Files(metaclass=ABCMeta):

    def __init__(self, path_file=None):
        self.path = path_file

    @abstractmethod
    def list_files(self):
        list_dir = os.listdir(self.path)
        list_file = list()
        for file in list_dir:
            if os.path.isfile(os.path.join(self.path, file)):
                if file.lower().endswith(self.extension.lower()):
                    name, ext = os.path.splitext(file)
                    if self.source == "ONS":
                        list_file.append(name)
                    elif self.source == "ANA":
                        if name.split("_T_")[0] == self.typesData[self.type_data][1]:
                            list_file.append(name)
        if len(list_file) == 1:
            list_file = list_file[0]
        return list_file



"""
Created on 21 de mar de 2018

@author: clebson
"""
import pandas as pd
import os

from hydrocomp.files import Files


class FileWrite(Files):

    def __init__(self, df, name=None):
        self.df = df
        self.name = name

    def excel(self):
        writer = pd.ExcelWriter('%s.xlsx' % self.name)
        self.df.to_excel(writer)
        writer.save()

    def csv(self):
        self.df.to_csv('%s.csv' % self.name)

    def json(self):
        self.df.to_json('%s.json' % self.name)

    def txt(self, name_dir, hour=True):
        if hour:
            pass
        else:
            self.df = self.df.resample('H').sum()
        bool = self.df.isnull()
        cont = 1
        arq2 = open(os.path.join(os.getcwd(), 'interplu'+'.txt'), 'w')
        arq2.write('{:>20}{:>23}{:>13}\n'.format('codigo', 'long dec', 'lat dec'))
        for i in self.df:
            data = pd.to_datetime('1/1/1977')
            name_arq = '{:08}.txt'.format(cont)
            cont += 1
            arq2.write('{:>20}{:>23}{:>13}\n'.format(name_arq, i[0], i[1]))
            arq = open(os.path.join(os.path.join(os.getcwd(),
                                                 name_dir), name_arq), 'w')
            for j in self.df[i].index:
                if bool[i][j]:
                    j =- 1
                else:
                    j = self.df[i][j]
                arq.write('{:>6}{:>6}{:>6}{:>12}\n'.format(data.day, data.month,
                                                           data.year, j))
                data += pd.DateOffset(days=1)
            arq.close()
        arq2.close()



import os
import pandas as pd
import plotly.figure_factory as FF
import plotly.graph_objs as go

from abc import abstractmethod, ABCMeta

from hydrocomp.files import ana, ons
from hydrocomp.graphics.gantt import Gantt


class SeriesBuild(metaclass=ABCMeta):

    sources = {
        "ONS": ons.Ons,
        "ANA": ana.Ana
    }

    def __init__(self, data=None, path=None, station=None, source=None, *args, **kwargs):
        self.path = path
        if data is not None:
            try:
                if type(station) == list():
                    self.station = None
                    self.__return_df(data)
                elif station is not None:
                    self.station = station
                    self.__return_df(data)
                    self.data = self.data.rename(columns={self.data.columns[0]: self.station}).sort_index()
                elif len(data.columns) == 1:
                    self.__return_df(data)
                    self.station = self.data.columns[0]
                else:
                    self.__return_df(data)
                    self.station = None
            except KeyError:
                self.station = None
                self.__return_df(data)
        else:
            if type(station) != list():
                self.station = station
            else:
                self.station = None
            if source in self.sources:
                self.source = source
                read = self.sources[self.source](path_file=self.path, station=self.station, *args, **kwargs)
                self.data = read.data.sort_index()
            else:
                raise KeyError('Source not supported!')
        if self.data.size == 0:
            self.date_start, self.date_end = None, None
        else:
            self.date_start, self.date_end = self.__start_and_end()
            _data = pd.DataFrame(index=pd.date_range(start=self.date_start, end=self.date_end))
            self.data = _data.combine_first(self.data[self.date_start:self.date_end])

    def __return_df(self, data):
        if type(data) is type(pd.Series()):
            self.data = pd.DataFrame(data)
        else:
            self.data = data

    @abstractmethod
    def month_start_year_hydrologic(self):
        pass

    @abstractmethod
    def hydrogram(self, title, save=False, width=None, height=None, size_text=None):
        pass

    def __start_and_end(self):
        try:
            boolean = self.data.dropna(axis=0, how='all')
        except AttributeError:
            boolean = self.data
        date = boolean.index
        return date[0], date[-1]

    def __str__(self):
        """
        """
        return self.data.__repr__()

    def __getitem__(self, val):
        """
        """
        return self.__class__(data=self.data[val].copy())

    def date(self, date_start=None, date_end=None):
        """
        """
        if date_start is not None and date_end is not None:
            date_start = pd.to_datetime(date_start, dayfirst=True)
            date_end = pd.to_datetime(date_end, dayfirst=True)
            self.data = self.data.loc[date_start:date_end]
        elif date_start is not None:
            date_start = pd.to_datetime(date_start, dayfirst=True)
            self.data = self.data.loc[date_start:]
        elif date_end is not None:
            date_end = pd.to_datetime(date_end, dayfirst=True)
            self.data = self.data.loc[:date_end].copy()

    def less_period(self, data):
        """
        """
        aux = list()
        list_start = list()
        list_end = list()
        gantt_bool = data.isnull()
        for i in gantt_bool.index:
            if ~gantt_bool.loc[i]:
                aux.append(i)
            elif len(aux) > 2 and gantt_bool.loc[i]:
                list_start.append(aux[0])
                list_end.append(aux[-1])
                aux = []
        if len(aux) > 0:
            list_start.append(aux[0])
            list_end.append(aux[-1])
        dic = {'Start': list_start, 'Finish': list_end}
        return pd.DataFrame(dic)

    def summary(self):
        """
        """
        return self.data.describe()

    def get_year(self, year):
        """
        Seleciona todos os dados referente ao ano.
        """
        return self.__getitem__(year)

    def get_month(self, month):
        """
        Selecina todos os dados referente ao mês
        """
        return self.data.groupby(lambda x: x.month).get_group(month)

    def mean(self):
        """
        """
        return self.data.mean().values

    def std(self):
        """
        """
        return self.data.std()

    def quantile(self, percentile):
        return self.data.quantile(percentile).values

    def gantt(self, title=None, size_text=14):
        cont = 0
        df = pd.DataFrame(columns=['Task', 'Start', 'Finish', 'Description', 'IndexCol'])
        for i in self.data:
            df, cont = Gantt(self.data[i]).get_gantt(df, self.less_period(self.data[i]), cont)
        colors = ['#000000', '#778899']

        fig = FF.create_gantt(df, colors=colors, index_col='IndexCol', group_tasks=True)

        fig.layout.xaxis.title = "Data"
        fig.layout.title = dict(text=title, x=0.5, xanchor='center', y=0.9, yanchor='top',
                                font=dict(family='Courier New, monospace', color='#7f7f7f', size=size_text + 6))
        fig.layout.font = dict(family='Courier New, monospace', size=size_text, color='#7f7f7f')
        fig.layout.plot_bgcolor = 'rgba(0,0,0,0)'
        return fig, df



from setuptools import setup, find_packages
import os

rootpath = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    return open(os.path.join(rootpath, *parts), 'r').read()


long_description = '{}'.format(read('README.md'))

with open('requirements.txt') as f:
    require = f.readlines()
install_requires = [r.strip() for r in require]

setup(
    name='hydrocomp',
    version='0.1.7',
    include_package_data=True,
    pacotes=find_packages('src'),
    long_description=long_description,
    classifiers=['Development Status :: 1 - Planning',
                 'Environment :: Console',
                 'Intended Audience :: Science/Research',
                 'Intended Audience :: Developers',
                 'Intended Audience :: Education',
                 'License :: OSI Approved :: MIT License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Education',
                 ],
    url='https://github.com/clebsonpy/HydroComp',
    license='MIT License',
    author='Clebson Farias',
    author_email='clebson2007.farias@gmail.com',
    keywords='hydrology statistic iha flow',
    description='Developed for hydrological studies',
    install_requires=install_requires,
    packages=['hydrocomp'],
)



from abc import ABCMeta, abstractmethod

class DistributionBiuld(object, metaclass=ABCMeta):

    def __init__(self, title, shape, location, scale):
        self.title = title
        self.shape = shape
        self.location = location
        self.scale = scale

    def plot(self, type_function):
        if type_function == 'density':
            return self.density()
        elif type_function == 'cumulative':
            return self.cumulative()

    def _data(self, type_function):
        if type_function == 'density':
            return self._data_density()
        elif type_function == 'cumulative':
            return self._data_cumulative()

    @abstractmethod
    def _data_cumulative(self):
        pass

    @abstractmethod
    def _data_density(self):
        pass

    @abstractmethod
    def cumulative(self):
        pass

    @abstractmethod
    def density(self):
        pass



import numpy as np

from hydrocomp import statistic as e
from abc import ABCMeta, abstractmethod


class StatsBuild(metaclass=ABCMeta):

    parameter = None
    dist = None

    def __init__(self, data=None, *args, **kwargs):
        for i in kwargs:
            self.parameter[i] = kwargs[i]

        if data is None:
            if None in self.parameter.values():
                raise e.DataNotExist("Parâmetros não  informados", 12)
        else:
            self.data = data

    def plot_pdf(self):
        pass

    def plot_cdf(self):
        pass

    @abstractmethod
    def mvs(self):
        pass

    @abstractmethod
    def mml(self):
        pass

    @abstractmethod
    def rvs(self, n):
        pass

    def probs(self, x):
        if self.dist is None:
            raise e.DistributionNotExist('Distribuição não existe', 51)
        else:
            if type(x) is list:
                return [self.probs(i) for i in x]
            return self.dist.cdf(x)

    def values(self, p):
        if self.dist is None:
            raise e.DistributionNotExist('Distribuição não existe', 51)
        else:
            if type(p) is list or type(p) is np.ndarray:
                return [self.values(i) for i in p]
            return self.dist.ppf(p)

    def interval(self, alpha):
        if self.dist is None:
            raise e.DistributionNotExist('Distribuição não existe', 51)
        else:
            return self.dist.interval(alpha)

    def rvs(self, n):
        return self.values(np.random.random(n))



import os
import calendar as cal
import pandas as pd
from hydrocomp.series.exceptions import StationError

from hydrocomp.statistic.pearson3 import Pearson3
from hydrocomp.series.series_build import SeriesBuild
from hydrocomp.series.parcial import Parcial
from hydrocomp.series.maximum import Maximum
from hydrocomp.series.minimum import Minimum
from hydrocomp.graphics.hydrogram_clean import HydrogramClean
from hydrocomp.graphics.hydrogram_by_year import HydrogramYear
from hydrocomp.graphics.permanence_curve import PermanenceCurve


class Flow(SeriesBuild):
    type_data = 'FLUVIOMÉTRICO'

    def __init__(self, data=None, path_file=None, station=None, source=None, *args, **kwargs):
        super().__init__(data=data, path=path_file, station=station, source=source,
                         type_data=self.type_data, *args, **kwargs)
        self.month_num_flood = 1
        self.month_abr_flood = 'AS-JAN'
        self.month_num_drought = 12
        self.month_abr_drought = 'AS-DEC'

    def month_start_year_hydrologic(self):
        if self.station is None:
            mean_month = pd.DataFrame([self.data.loc[self.data.index.month == i].mean() for i in range(1, 13)])
            month_start_year_hydrologic = 1 + mean_month.idxmin().values[0]
            month_start_year_hydrologic_abr_flood = cal.month_abbr[month_start_year_hydrologic].upper()
        else:
            data = pd.DataFrame(self.data[self.station])
            mean_month = pd.DataFrame([data.loc[data.index.month == i].mean() for i in range(1, 13)])
            month_start_year_hydrologic = 1 + mean_month.idxmin().values[0]
            month_start_year_hydrologic_abr_flood = cal.month_abbr[month_start_year_hydrologic].upper()

        self.month_num_flood = month_start_year_hydrologic
        self.month_abr_flood = 'AS-%s' % month_start_year_hydrologic_abr_flood

        self.month_num_drought = month_start_year_hydrologic - 6
        month_start_year_hydrologic_abr_drought = cal.month_abbr[self.month_num_drought].upper()
        self.month_abr_drought = 'AS-%s' % month_start_year_hydrologic_abr_drought
        return self.month_num_flood, self.month_abr_flood, self.month_num_drought, self.month_abr_drought

    def minimum(self):
        minimum = Minimum(obj=self, station=self.station)

        return minimum

    def maximum(self):
        maximum = Maximum(obj=self, station=self.station)

        return maximum

    def parcial(self, type_threshold, type_event, type_criterion, value_threshold, **kwargs):
        parcial = Parcial(station=self.station, obj=self, type_threshold=type_threshold, type_event=type_event,
                          type_criterion=type_criterion, value_threshold=value_threshold, **kwargs)

        return parcial

    def simulation_withdraw(self, criterion, rate, months=None, value=None):
        if type(months) is not list and months is not None:
            raise TypeError

        if criterion == 'q90':
            data = self.data.copy()
            if value is None:
                withdraw = (self.quantile(percentile=0.1) * (rate / 100)).values[0]
            else:
                withdraw = (value * (rate / 100)).values[0]

            if months is None:
                name = "withdraw_{}_{}_{}".format(rate, criterion, 'all')
                data.rename(columns={self.station: name}, inplace=True)
                data = data - withdraw
                data.loc[data[name] < 0, name] = 0
                return data
            else:
                name = "withdraw_{}_{}_{}".format(rate, criterion, 'months')
                data.rename(columns={self.station: name}, inplace=True)
                for i in self.data[self.station].index:
                    if i.month in months:
                        data.at[i, name] = self.data[self.station][i] - withdraw
                        data.loc[data[name] < 0, name] = 0
                return data
        else:
            return None

    def hydrogram(self, title, threshold=None, save=False, width=None, height=None, y_title='Vazão (m³/s)',
                  x_title='Data', size_text=16, color=None):
        if self.station is None:
            hydrogram = HydrogramClean(self.data, threshold=threshold, width=width, height=height, size_text=size_text,
                                       title=title, y_title=y_title, x_title=x_title, color=color)
            fig, data = hydrogram.plot()
        else:
            hydrogram = HydrogramClean(self.data[self.station], threshold=threshold, width=width, height=height,
                                       size_text=size_text, title=title, y_title=y_title, x_title=x_title, color=color)
            fig, data = hydrogram.plot()
        return fig, data

    def hydrogram_year(self, title="", threshold=None, width=None, height=None, size_text=16):
        self.month_start_year_hydrologic()
        idx = [i for i in self.data.index if i.month == 2 and i.day == 29]
        data = self.data.drop(index=idx)
        data = data.groupby(pd.Grouper(freq=self.month_abr))
        hydrogram = HydrogramYear(data=data, threshold=threshold, width=width, height=height, title=title,
                                  size_text=size_text)
        fig, data = hydrogram.plot()
        return fig, data

    def permanence_curve(self, width=None, height=None, size_text=None, title=None):
        if self.station is None:
            raise StationError
        permanence = PermanenceCurve(self.data[self.station], width=width, height=height, size_text=size_text,
                                     title=title)
        fig, data = permanence.plot()
        return fig, data

    def get_month_name(self):
        months = {1: 'Janeiro', 2: 'Fevereiro', 3: 'Março', 4: 'Abril', 5: 'Maio', 6: 'Junho', 7: 'Julho', 8: 'Agosto',
                  9: 'Setembro', 10: 'Outubro', 11: 'Novembro', 12: 'Dezembro'}
        return months[self.month_num]

    def flow_min(self, method):
        if method == 'q90':
            return self.quantile(0.1)
        elif method == 'q95':
            return self.quantile(0.05)
        elif method == 'q710':
            qmin = self.data.rolling(7).mean().groupby(pd.Grouper(freq='AS-JAN')).min()
            prop = 1 / 10
            dist = Pearson3(qmin[self.station].values)
            dist.mml()
            return dist.values(prop)
